@@ -37,18 +37,32 @@ char *global_options_directory = NULL;
 gboolean global_options_replace = FALSE;
 gboolean global_options_fullscreen = FALSE;
 
+static void correct_scheme(const char *bad_name, const char *good_name)
+{
+	char *val = global_options_lookup_string(bad_name);
+
+	if (!val)
+		return;
+	if (global_options_lookup_string(good_name))
+		return;
+	/*g_print("Converting %s to %s (%s)\n", bad_name, good_name, val);*/
+	options_set_string(global_options, good_name, val);
+}
+
+inline static void correct_schemes(void)
+{
+	correct_scheme("colour-scheme", "colour_scheme");
+	correct_scheme("shortcut-scheme", "shortcut_scheme");
+}
+
 static void global_options_reset(void)
 {
-	if (global_options_commandv)
-	{
-		g_strfreev(global_options_commandv);
-		global_options_commandv = NULL;
-	}
 	g_free(global_options_directory);
 	global_options_directory = NULL;
 	global_options_replace = FALSE;
 	global_options_fullscreen = FALSE;
 	options_reload_keyfile(global_options);
+	correct_schemes();
 }
 
 static void global_options_show_usage(const gchar *option_name,
@@ -184,6 +198,11 @@ gboolean global_options_preparse_argv_for_execute(int *argc, char **argv,
 {
 	int n;
 
+	if (global_options_commandv)
+	{
+		g_strfreev(global_options_commandv);
+		global_options_commandv = NULL;
+	}
 	for (n = 0; n < *argc; ++n)
 	{
 		if (!strcmp(argv[n], "--execute") || !strcmp(argv[n], "-e"))
@@ -245,19 +264,6 @@ static void global_options_parse_argv(int argc, char **argv)
 	}
 }
 
-static void correct_scheme(const char *bad_name, const char *good_name)
-{
-	char *val = global_options_lookup_string(bad_name);
-
-	if (!val)
-		return;
-	if (global_options_lookup_string(good_name))
-	{
-		return;
-	}
-	options_set_string(global_options, good_name, val);
-}
-
 void global_options_init(int argc, char **argv)
 {
 	static gboolean already = FALSE;
@@ -271,8 +277,7 @@ void global_options_init(int argc, char **argv)
 		global_options_init_appdir(argc, argv);
 		/* Need to get appdir before trying to load any options files */
 		global_options = options_open("Global", "roxterm options");
-		correct_scheme("colour-scheme", "colour_scheme");
-		correct_scheme("shortcut-scheme", "shortcut_scheme");
+		correct_schemes();
 	}
 	global_options_parse_argv(argc, argv);
 	if (!already)
