@@ -44,7 +44,7 @@ static gboolean run_via_dbus(int argc, char **argv)
 {
 	DBusMessage *message;
 	int n;
-	gboolean result = FALSE;
+	gboolean result = TRUE;
 
 	message = rtdbus_method_new(ROXTERM_DBUS_NAME, ROXTERM_DBUS_OBJECT_PATH,
 			ROXTERM_DBUS_INTERFACE,
@@ -52,7 +52,8 @@ static gboolean run_via_dbus(int argc, char **argv)
 	for (n = 0; message && n < argc; ++n)
 	{
 		message = rtdbus_append_args(message,
-				DBUS_TYPE_STRING, RTDBUS_ARG(argv[n]), DBUS_TYPE_INVALID);
+				DBUS_TYPE_STRING, RTDBUS_ARG(argv[n]),
+				DBUS_TYPE_INVALID);
 	}
 	if (!message)
 		return FALSE;
@@ -142,7 +143,6 @@ int main(int argc, char **argv)
 	gboolean preparse_ok;
 
 	g_set_application_name(_("roxterm"));
-
 	preparse_ok = global_options_preparse_argv_for_execute(&argc, argv, FALSE);
 	gtk_init(&argc, &argv);
 	if (!preparse_ok)
