@@ -50,6 +50,7 @@ struct MultiWin {
 	guint ntabs;
 	GList *tabs;
 	MultiTab *current_tab;
+	GtkPositionType tab_pos;
 	gpointer user_data_template;
 	gulong destroy_handler;
 	gboolean ignore_tab_selections;
@@ -64,7 +65,10 @@ struct MultiWin {
 	int child_width, child_height;	/* Need to store size of child widget before
 									   removing or adding a tab so we can
 									   correct size afterwards */
-	gboolean ignore_size_allocate;
+	int ignore_size_allocate;		/* 0 = Don't ignore
+									   1 = Ignore
+									   2 = Special case after tab label has
+									       changed */
 	MultiTabSelectionHandler tab_selection_handler;
 	gboolean menu_bar_set;		/* Menu bar can be configured either from
 								   profile when opening a window, or when user
@@ -107,6 +111,7 @@ static MultiWinSizeFunc multi_win_size_func;
 static MultiTabToNewWindowHandler multi_tab_to_new_window_handler;
 static MultiWinZoomHandler multi_win_zoom_handler;
 static MultiWinGetDisableMenuShortcuts multi_win_get_disable_menu_shortcuts;
+static MultiWinInitialTabs multi_win_initial_tabs;
 
 
 static gboolean multi_win_notify_tab_removed(MultiWin *, MultiTab *);
@@ -142,7 +147,8 @@ void multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
 	MultiWinGeometryFunc geometry_func, MultiWinSizeFunc size_func,
 	MultiTabToNewWindowHandler tab_to_new_window_handler,
 	MultiWinZoomHandler zoom_handler,
-	MultiWinGetDisableMenuShortcuts get_disable_menu_shortcuts)
+	MultiWinGetDisableMenuShortcuts get_disable_menu_shortcuts,
+	MultiWinInitialTabs initial_tabs)
 {
 	multi_tab_filler = filler;
 	multi_tab_destructor = destructor;
@@ -152,6 +158,7 @@ void multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
 	multi_tab_to_new_window_handler = tab_to_new_window_handler;
 	multi_win_zoom_handler = zoom_handler;
 	multi_win_get_disable_menu_shortcuts = get_disable_menu_shortcuts;
+	multi_win_initial_tabs = initial_tabs;
 }
 
 MultiTab *multi_tab_new(MultiWin * parent, gpointer user_data_template)
@@ -207,13 +214,13 @@ void multi_tab_delete(MultiTab * tab)
 {
 	MultiWin *win = tab->parent;
 
-	win->ignore_size_allocate = TRUE;
+	win->ignore_size_allocate = 1;
 	win->ignore_tabs_moving = TRUE;
 	multi_tab_delete_without_notifying_parent(tab, TRUE);
 	if (!multi_win_notify_tab_removed(win, tab))
 	{
 		win->ignore_tabs_moving = FALSE;
-		win->ignore_size_allocate = FALSE;
+		win->ignore_size_allocate = 0;
 	}
 }
 
@@ -228,6 +235,10 @@ static void multi_tab_set_name_or_title(MultiTab * tab, const char *title)
 
 	if (tab->label)
 	{
+		GtkPositionType tab_pos = tab->parent->tab_pos;
+
+		if (tab_pos == GTK_POS_LEFT || tab_pos == GTK_POS_RIGHT)
+			win->ignore_size_allocate = 2;
 		gtk_label_set_text(GTK_LABEL(tab->label), title);
 	}
 	win->ignore_toggles = TRUE;
@@ -260,7 +271,8 @@ void multi_tab_set_title(MultiTab * tab, const char *title)
 		gtk_window_set_title(GTK_WINDOW(win->gtkwin),
 			STR_EMPTY(tab->title));
 	}
-	multi_tab_set_name_or_title(tab, title);
+	if (!tab->name)
+		multi_tab_set_name_or_title(tab, title);
 }
 
 void multi_tab_set_name(MultiTab * tab, const char *name)
@@ -303,7 +315,7 @@ MultiTab *multi_tab_get_tab_under_pointer(int x, int y)
 		return NULL;
 	if (win->ntabs == 1)
 		return win->tabs->data;
-	tab_pos = gtk_notebook_get_tab_pos(GTK_NOTEBOOK(win->notebook));
+	tab_pos = win->tab_pos;
 	vert_tabs = (tab_pos == GTK_POS_LEFT || tab_pos == GTK_POS_RIGHT);
 	for (tab_node = win->tabs; tab_node; tab_node = g_list_next(tab_node))
 	{
@@ -429,7 +441,7 @@ void multi_tab_remove_from_parent(MultiTab *tab, gboolean notify_only)
 	win->ignore_tabs_moving = TRUE;
 	if (!notify_only)
 	{
-		win->ignore_size_allocate = TRUE;
+		win->ignore_size_allocate = 1;
 		g_object_ref(tab->widget);
 		gtk_notebook_remove_page(GTK_NOTEBOOK(win->notebook),
 				multi_tab_get_page_num(tab));
@@ -437,7 +449,7 @@ void multi_tab_remove_from_parent(MultiTab *tab, gboolean notify_only)
 	multi_tab_remove_menutree_items(win, tab);
 	if (!multi_win_notify_tab_removed(win, tab))
 	{
-		win->ignore_size_allocate = FALSE;
+		win->ignore_size_allocate = 0;
 		win->ignore_tabs_moving = FALSE;
 	}
 	tab->parent = NULL;
@@ -520,7 +532,7 @@ multi_win_set_size(MultiWin * win, GtkWidget * child, int width, int height)
 		gtk_widget_set_size_request(child, width, height);
 		return;
 	}
-	win->ignore_size_allocate = TRUE;
+	win->ignore_size_allocate = 1;
 	/* If window is already showing it's more complicated.
 	 * gtk_widget_set_size_request() on child doesn't perform corresponding
 	 * resize on parent window, so we need to resize window instead. It may
@@ -529,7 +541,7 @@ multi_win_set_size(MultiWin * win, GtkWidget * child, int width, int height)
 	gtk_widget_set_size_request(child, 10000, 10000);
 	difference_between_child_and_parent(child, win->gtkwin, &dw, &dh);
 	gtk_window_resize(GTK_WINDOW(win->gtkwin), width + dw, height + dh);
-	win->ignore_size_allocate = FALSE;
+	win->ignore_size_allocate = 0;
 }
 
 static void multi_win_set_geometry_hints_for_tab(MultiWin * win, MultiTab * tab)
@@ -607,7 +619,7 @@ void multi_win_set_show_menu_bar(MultiWin * win, gboolean show, gboolean resize)
 	if (!resize && win->menu_bar_set)
 		return;
 	win->menu_bar_set = TRUE;
-	win->ignore_size_allocate = TRUE;
+	win->ignore_size_allocate = 1;
 	if (show)
 		add_menu_bar(win);
 	else
@@ -619,7 +631,7 @@ void multi_win_set_show_menu_bar(MultiWin * win, gboolean show, gboolean resize)
 		multi_win_set_size(win, win->current_tab->active_widget,
 			win->child_width, win->child_height);
 	}
-	win->ignore_size_allocate = FALSE;
+	win->ignore_size_allocate = 0;
 }
 
 static void multi_win_set_fullscreen(MultiWin *win, gboolean fullscreen)
@@ -652,16 +664,33 @@ static void
 multi_win_size_allocate(GtkWidget * widget, GtkAllocation * allocation,
 	MultiWin * win)
 {
-	if (!win->ignore_size_allocate)
-	{
-		GList *link;
+	GList *link;
+	MultiTab *tab;
 
-		for (link = win->tabs; link; link = g_list_next(link))
-		{
-			(*multi_win_size_func) (((MultiTab *) link->data)->user_data,
-				MultiWin_UpdateConfiguredSize,
+	switch (win->ignore_size_allocate)
+	{
+		case 0:
+			for (link = win->tabs; link; link = g_list_next(link))
+			{
+				tab = (MultiTab *) link->data;
+				(*multi_win_size_func) (tab->user_data,
+					MultiWin_UpdateConfiguredSize,
+					&win->child_width, &win->child_height);
+			}
+			break;
+		case 1:
+			break;
+		case 2:
+			tab = win->current_tab;
+			if (!tab)
+				win->ignore_size_allocate = 0;
+			g_return_if_fail(tab != NULL);
+			(*multi_win_size_func) (tab->user_data,
+				MultiWin_GetTargetSize,
 				&win->child_width, &win->child_height);
-		}
+			multi_win_set_size(win, tab->active_widget,
+				win->child_width, win->child_height);
+			break;
 	}
 }
 
@@ -698,14 +727,36 @@ multi_win_page_switched(GtkNotebook * notebook, GtkNotebookPage * page,
 	multi_win_select_tab(win, tab);
 }
 
+static void tab_set_name_from_clipboard_callback(GtkClipboard *clp,
+		const gchar *text, gpointer data)
+{
+	MultiTab *tab = (MultiTab *) data;
+	
+	if (text == NULL || tab == NULL)
+		return;
+	multi_tab_set_name(tab, text);
+}
+
 #if DO_OWN_TAB_DRAGGING
 static gboolean tab_clicked_handler(GtkWidget *widget,
 		GdkEventButton *event, MultiTab *tab)
 {
-	if ((event->button == 1 || event->button == 2)
-			&& event->type == GDK_BUTTON_PRESS)
+	if (event->type != GDK_BUTTON_PRESS)
+		return FALSE;
+	
+	switch (event->button)
 	{
-		tab_drag_press(tab->tab_drag, event->button, event->x, event->y);
+		case 2:
+			gtk_clipboard_request_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
+									   tab_set_name_from_clipboard_callback,
+									   tab);
+			break;
+		case 1:
+		case 3:
+			tab_drag_press(tab->tab_drag, event->button, event->x, event->y);
+			break;
+		default:
+			break;
 	}
 	return FALSE;
 }
@@ -788,7 +839,8 @@ static GtkNotebook *multi_win_notebook_creation_hook(GtkNotebook *source,
 		&win->child_width, &win->child_height);
 	multi_tab_remove_from_parent(tab, TRUE);
 	win = multi_win_new_blank(multi_win_get_shortcut_scheme(win),
-				((MultiWin *) data)->zoom_index, multi_tab_get_user_data(tab));
+				((MultiWin *) data)->zoom_index, multi_tab_get_user_data(tab),
+				win->tab_pos);
 	multi_win_add_tab(win, tab, -1, TRUE);
 	gtk_widget_show_all(win->notebook);
 	/* Defer showing till page_added_callback because we can't set size
@@ -805,9 +857,13 @@ static void multi_win_close_tab_clicked(GtkWidget *widget, MultiTab *tab)
 
 static void multi_win_new_window_action(MultiWin * win)
 {
+	GtkPositionType tab_pos = GTK_POS_TOP;
+	int num_tabs = 1;
+
+	multi_win_initial_tabs(win->current_tab->user_data, &tab_pos, &num_tabs);
 	multi_win_new(win->shortcuts, win->zoom_index,
-		win->current_tab ? win->current_tab->user_data :
-		win->user_data_template);
+		      win->current_tab ? win->current_tab->user_data :
+		      win->user_data_template, num_tabs, tab_pos);
 }
 
 static void multi_win_new_tab_action(MultiWin * win)
@@ -851,6 +907,36 @@ static void multi_win_name_tab_action(MultiWin * win)
 	gtk_widget_destroy(dialog_w);
 }
 
+static void multi_win_set_window_title_action( MultiWin * win )
+{
+	MultiTab *tab = win->current_tab;
+	g_return_if_fail(tab);
+	GtkWidget *dialog_w = gtk_dialog_new_with_buttons(_("Set Window Title"),
+			GTK_WINDOW(win->gtkwin),
+			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
+			NULL);
+	GtkDialog *dialog = GTK_DIALOG(dialog_w);
+	GtkWidget *title_w = gtk_entry_new();
+	GtkEntry *title_e = GTK_ENTRY(title_w);
+	const char *title = tab->title;
+
+	gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
+	gtk_box_pack_start(GTK_BOX(dialog->vbox), title_w, TRUE, TRUE, 8);
+	gtk_entry_set_activates_default(title_e, TRUE);
+	gtk_entry_set_text(title_e, title ? title : "");
+	gtk_widget_show_all(dialog_w);
+	if (gtk_dialog_run(dialog) == GTK_RESPONSE_APPLY)
+	{
+		title = gtk_entry_get_text(title_e);
+		if (title && !title[0])
+			title = NULL;
+		multi_tab_set_title(tab, title);
+	}
+	gtk_widget_destroy(dialog_w);
+}
+
 static void multi_win_next_tab_action(MultiWin * win)
 {
 	if (win->wrap_switch_tab && multi_win_at_last_tab(win))
@@ -983,6 +1069,8 @@ static void multi_win_connect_actions(MultiWin * win)
 		(multi_win_next_tab_action), win, NULL, NULL);
 	multi_win_menu_connect_swapped(win, MENUTREE_TABS_NAME_TAB, G_CALLBACK
 		(multi_win_name_tab_action), win, NULL, NULL);
+	multi_win_menu_connect_swapped(win, MENUTREE_EDIT_SET_WINDOW_TITLE,
+		G_CALLBACK(multi_win_set_window_title_action), win, NULL, NULL);
 	multi_win_menu_connect(win, MENUTREE_VIEW_SHOW_MENUBAR, G_CALLBACK
 		(multi_win_toggle_show_menubar_action), win, NULL, NULL);
 	multi_win_menu_connect(win, MENUTREE_VIEW_FULLSCREEN, G_CALLBACK
@@ -1020,7 +1108,7 @@ void multi_win_show(MultiWin *win)
 }
 
 MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
-		gpointer user_data_template)
+		gpointer user_data_template, GtkPositionType tab_pos)
 {
 	MultiWin *win = g_new0(MultiWin, 1);
 	gboolean disable_menu_shortcuts =
@@ -1036,6 +1124,7 @@ MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
 	}
 #endif
 
+	win->tab_pos = tab_pos;
 	win->scroll_bar_pos = MultiWinScrollBar_Query;
 	win->user_data_template = user_data_template;
 	if (zoom_index < 0)
@@ -1100,13 +1189,20 @@ MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
 }
 
 MultiWin *multi_win_new_full(Options *shortcuts, int zoom_index,
-		gpointer user_data_template, const char *geom, gboolean fullscreen)
+			     gpointer user_data_template, const char *geom,
+			     gboolean fullscreen, int numtabs, GtkPositionType tab_pos)
 {
 	MultiWin *win = multi_win_new_blank(shortcuts, zoom_index,
-			user_data_template);
+			user_data_template, tab_pos);
 
-	multi_tab_new(win, user_data_template);
+	win->tab_pos = tab_pos;
+	while(numtabs--)
+		multi_tab_new(win, user_data_template);
+
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(win->notebook), 0);
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(win->notebook), tab_pos);
 	multi_win_shade_menus_for_tabs(win);
+
 	if (geom)
 	{
 		gtk_window_parse_geometry(GTK_WINDOW(win->gtkwin), geom);
@@ -1213,9 +1309,10 @@ static gboolean multi_win_notify_tab_removed(MultiWin * win, MultiTab * tab)
 
 /* Creates the label widget for a tab. tab->label is the GtkLabel containing
  * the text; the return value is the top-level container. */
-static GtkWidget *make_tab_label(MultiTab *tab)
+static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
 {
-	GtkWidget *hbox = gtk_hbox_new(FALSE, 4);
+	GtkWidget *hbox;
+	void (*packer)(GtkBox *, GtkWidget *, gboolean, gboolean, guint);
 #if DO_OWN_TAB_DRAGGING
 	GtkWidget *ebox = gtk_event_box_new();
 #endif
@@ -1227,7 +1324,22 @@ static GtkWidget *make_tab_label(MultiTab *tab)
 	int w, h;
 
 	tab->label = gtk_label_new(multi_tab_get_name_or_title(tab));
-	gtk_label_set_ellipsize(GTK_LABEL(tab->label), PANGO_ELLIPSIZE_START);
+	switch (tab_pos)
+	{
+		case GTK_POS_LEFT:
+		case GTK_POS_RIGHT:
+			hbox = gtk_vbox_new(FALSE, 4);
+			packer = gtk_box_pack_end;
+			gtk_label_set_ellipsize(GTK_LABEL(tab->label),
+					PANGO_ELLIPSIZE_NONE);
+			break;
+		default:
+			hbox = gtk_hbox_new(FALSE, 4);
+			packer = gtk_box_pack_start;
+			gtk_label_set_ellipsize(GTK_LABEL(tab->label),
+					PANGO_ELLIPSIZE_MIDDLE);
+			 break;
+	}
 	gtk_container_add(GTK_CONTAINER(close_button), image);
 	gtk_button_set_relief(GTK_BUTTON(close_button), GTK_RELIEF_NONE);
 	gtk_button_set_focus_on_click(GTK_BUTTON(close_button), FALSE);
@@ -1239,15 +1351,15 @@ static GtkWidget *make_tab_label(MultiTab *tab)
 	gtk_rc_style_unref (rcstyle);
 #if DO_OWN_TAB_DRAGGING
 	gtk_event_box_set_visible_window(GTK_EVENT_BOX(ebox), FALSE);
-	gtk_box_pack_start(GTK_BOX(hbox), ebox, TRUE, TRUE, 0);
+	packer(GTK_BOX(hbox), ebox, TRUE, TRUE, 0);
 	gtk_container_add(GTK_CONTAINER(ebox), tab->label);
 	tab->tab_drag = tab_drag_new(ebox, tab);
 	g_signal_connect(ebox, "button-press-event",
 			G_CALLBACK(tab_clicked_handler), tab);
 #else
-	gtk_box_pack_start(GTK_BOX(hbox), tab->label, TRUE, TRUE, 0);
+	packer(GTK_BOX(hbox), tab->label, TRUE, TRUE, 0);
 #endif
-	gtk_box_pack_start(GTK_BOX(hbox), close_button, FALSE, FALSE, 0);
+	packer(GTK_BOX(hbox), close_button, FALSE, FALSE, 0);
 	gtk_notebook_set_tab_label(GTK_NOTEBOOK(tab->parent->notebook),
 		tab->widget, hbox);
 	g_signal_connect(close_button, "clicked",
@@ -1278,7 +1390,7 @@ static void multi_tab_add_menutree_items(MultiWin * win, MultiTab * tab,
 static void multi_win_add_tab_to_notebook(MultiWin * win, MultiTab * tab,
 		int position)
 {
-	GtkWidget *label = make_tab_label(tab);
+	GtkWidget *label = make_tab_label(tab, win->tab_pos);
 
 	if (position == -1)
 	{
@@ -1308,7 +1420,7 @@ static void multi_win_add_tab(MultiWin * win, MultiTab * tab, int position,
 		gboolean notify_only)
 {
 	tab->parent = win;
-	win->ignore_size_allocate = TRUE;
+	win->ignore_size_allocate = 1;
 	win->ignore_tabs_moving = TRUE;
 	if (position == -1)
 		win->tabs = g_list_append(win->tabs, tab);
@@ -1336,7 +1448,7 @@ static void multi_win_add_tab(MultiWin * win, MultiTab * tab, int position,
 	multi_win_set_size(win, tab->active_widget,
 		win->child_width, win->child_height);
 	win->ignore_tabs_moving = FALSE;
-	win->ignore_size_allocate = FALSE;
+	win->ignore_size_allocate = 0;
 }
 
 GtkWidget *multi_win_get_widget(MultiWin * win)
@@ -1491,6 +1603,11 @@ void multi_win_foreach_tab(MultiWin *win, MultiWinForEachTabFunc func,
 	}
 }
 
+GtkPositionType multi_win_get_tab_pos(MultiWin *win)
+{
+	return win->tab_pos;
+}
+
 GList *multi_win_all = NULL;
 
 /* vi:set sw=4 ts=4 noet cindent cino= */
