@@ -19,7 +19,6 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-
 /* Hierarchy of multiple windows, each containing multiple tabs */
 
 #include "menutree.h"
@@ -84,6 +83,11 @@ typedef void (*MultiWinZoomHandler)(gpointer user_data, double zoom_factor,
 /* Called to find out whether to use menu shortcuts */
 typedef gboolean (*MultiWinGetDisableMenuShortcuts)(gpointer user_data);
 
+/* As number of tabs and tab position can be set by user we need another
+ * function to get these values */
+typedef void (*MultiWinInitialTabs)(gpointer user_data,
+		GtkPositionType *p_pos, int *p_num);
+
 /* Call to set up function hooks. See MultiTabFiller etc above.
  * menu_signal_connector is called each time a new window is created to give
  * the client a chance to connect its signal handlers; each handler will
@@ -92,7 +96,7 @@ void
 multi_tab_init(MultiTabFiller filler, MultiTabDestructor destructor,
 	MultiWinMenuSignalConnector menu_signal_connector,
 	MultiWinGeometryFunc, MultiWinSizeFunc, MultiTabToNewWindowHandler,
-	MultiWinZoomHandler, MultiWinGetDisableMenuShortcuts);
+	MultiWinZoomHandler, MultiWinGetDisableMenuShortcuts, MultiWinInitialTabs);
 
 /* Register a MultiTabSelectionHandler (see above) */
 void
@@ -150,34 +154,37 @@ MultiTab *multi_tab_get_tab_under_pointer(int x, int y);
 
 /* Create a new window with default settings, adding it to main list */
 MultiWin *multi_win_new_full(Options *shortcuts, int zoom_index,
-		gpointer user_data_template, const char *geom, gboolean full_screen);
+			     gpointer user_data_template, const char *geom,
+			     gboolean full_screen, int numtabs, GtkPositionType tab_pos);
 
 inline static MultiWin *multi_win_new_with_geom(Options *shortcuts,
-		int zoom_index, gpointer user_data_template, const char *geom)
+						int zoom_index, gpointer user_data_template,
+						const char *geom, int numtabs, GtkPositionType tab_pos)
 {
 	return multi_win_new_full(shortcuts, zoom_index,
-			user_data_template, geom, FALSE);
+				  user_data_template, geom, FALSE, numtabs, tab_pos);
 }
 
 inline static MultiWin *multi_win_new(Options *shortcuts, int zoom_index,
-		gpointer user_data_template)
+				      gpointer user_data_template, int numtabs, GtkPositionType tab_pos)
 {
 	return multi_win_new_full(shortcuts, zoom_index,
-			user_data_template, NULL, FALSE);
+				  user_data_template, NULL, FALSE, numtabs, tab_pos);
 }
 
 inline static MultiWin *multi_win_new_fullscreen(Options *shortcuts,
-		int zoom_index, gpointer user_data_template)
+						 int zoom_index, gpointer user_data_template,
+						 int numtabs, GtkPositionType tab_pos)
 {
 	return multi_win_new_full(shortcuts, zoom_index,
-			user_data_template, NULL, TRUE);
+				  user_data_template, NULL, TRUE, numtabs, tab_pos);
 }
 
 /* Create a new window without a new child tab, ready to have an existing
  * tab moved to it. Window isn't shown yet because we don't know how big we
  * want it to be until it contains a tab. */
 MultiWin *multi_win_new_blank(Options *shortcuts, int zoom_index,
-		gpointer user_data_template);
+		gpointer user_data_template, GtkPositionType tab_pos);
 
 void multi_win_show(MultiWin *win);
 
@@ -280,6 +287,8 @@ typedef void (*MultiWinForEachTabFunc)(MultiTab *tab, void *user_data);
 void multi_win_foreach_tab(MultiWin *, MultiWinForEachTabFunc,
 		gpointer user_data);
 
+GtkPositionType multi_win_get_tab_pos(MultiWin *win);
+
 /* List of all known windows - treat as read-only */
 extern GList *multi_win_all;
 
