@@ -172,6 +172,13 @@ inline static void roxterm_match_add(ROXTermData *roxterm, VteTerminal *vte,
 	g_array_append_val(roxterm->match_map, map);
 }
 
+/* There isn't a portable standard way of expressing a word boundary (POSIX
+ * 1003.2 had '[[:<:]]', GNU has '\<') so the most sensible thing to do seems
+ * to be to ignore them and allow valid URIs to have any old rubbish tacked on
+ * to the front of them (Claws Mail appears to do this).
+ */
+#define URLSTART ""
+
 #define URLMSGIDSET "[-A-Z\\^_a-z{|}~!\"#$%&'()*+,./0-9;:=?`]"
 #define URLEND "[^]'.}>) \t\r\n,\\\"]"
 #define URLHOST "[A-Za-z0-9][-A-Za-z0-9.]*"
@@ -187,27 +194,28 @@ inline static void roxterm_match_add(ROXTermData *roxterm, VteTerminal *vte,
 #define URLPATH "[/?][-A-Za-z0-9_$.+!*(),;:@&=?/~#%]*" URLEND
 
 static const char *full_urls[] = {
-	"\\<" URLSCHEME "//" URLEXTHOST,
-	"\\<" URLSCHEME "//" URLEXTHOST URLPATH
+	URLSTART URLSCHEME "//" URLEXTHOST,
+	URLSTART URLSCHEME "//" URLEXTHOST URLPATH
 };
 
 static const char *web_urls[] = {
-	"\\<www[0-9]*" URLFQDNTAIL,
-	"\\<www[0-9]*" URLFQDNTAIL URLPATH
+	URLSTART "www[0-9]*" URLFQDNTAIL,
+	URLSTART "www[0-9]*" URLFQDNTAIL URLPATH
 };
 
 static const char *ftp_urls[] = {
-	"\\<ftp[0-9]*" URLFQDNTAIL,
-	"\\<ftp[0-9]*" URLFQDNTAIL URLPATH
+	URLSTART "ftp[0-9]*" URLFQDNTAIL,
+	URLSTART "ftp[0-9]*" URLFQDNTAIL URLPATH
 };
 
 #define URL_EMAIL_USER "[-A-Za-z.]+"
 
-#define URL_NEWS "\\<news:" URLMSGIDSET "+@" URLEXTHOST
+#define URL_NEWS URLSTART "news:" URLMSGIDSET "+@" URLEXTHOST
 
 static const char *mailto_urls[] = {
-	"\\<" URL_EMAIL_USER "@" URLHOST,
-	"\\<mailto:" URL_EMAIL_USER "@" URLHOST "\\?[A-Za-z]+=" URLMSGIDSET "+"
+	URLSTART URL_EMAIL_USER "@" URLHOST,
+	URLSTART "mailto:" URL_EMAIL_USER "@" URLHOST \
+		"\\?[A-Za-z]+=" URLMSGIDSET "+"
 };
 
 static void roxterm_add_matches(ROXTermData *roxterm, VteTerminal *vte)
@@ -1415,20 +1423,14 @@ typedef enum {
 	roxterm_ChildExitRespawn
 } roxterm_ChildExitAction;
 
-static void roxterm_child_exited(VteTerminal *vte, ROXTermData *roxterm)
+static gboolean roxterm_post_child_exit(ROXTermData *roxterm)
 {
 	roxterm_ChildExitAction action =
 		options_lookup_int_with_default(roxterm->profile, "exit_action",
 				roxterm_ChildExitClose);
-	double delay = options_lookup_double(roxterm->profile, "exit_pause");
 
 	if (action == roxterm_ChildExitRespawn && roxterm->no_respawn)
 		action = roxterm_ChildExitClose;
-	if (delay)
-	{
-		vte_terminal_feed(vte, _("\nFinished.\n"), -1);
-		usleep((useconds_t) (delay * 1000000));
-	}
 	switch (action)
 	{
 		case roxterm_ChildExitClose:
@@ -1440,6 +1442,23 @@ static void roxterm_child_exited(VteTerminal *vte, ROXTermData *roxterm)
 			roxterm_run_command(roxterm, VTE_TERMINAL(roxterm->widget));
 			break;
 	}
+	return FALSE;
+}
+
+static void roxterm_child_exited(VteTerminal *vte, ROXTermData *roxterm)
+{
+	double delay = options_lookup_double(roxterm->profile, "exit_pause");
+
+	if (delay)
+	{
+		vte_terminal_feed(vte, _("\n.\n"), -1);
+		g_timeout_add((guint) (delay * 1000.0),
+				(GSourceFunc) roxterm_post_child_exit, roxterm);
+	}
+	else
+	{
+		roxterm_post_child_exit(roxterm);
+	}
 }
 
 inline static ROXTermData *roxterm_from_menutree(MenuTree *mtree)
@@ -2465,15 +2484,41 @@ static gboolean check_geom_offsets(char *g)
 	return TRUE;
 }
 
+static GtkPositionType roxterm_get_tab_pos(const ROXTermData *roxterm)
+{
+	switch (options_lookup_int_with_default(roxterm->profile, "tab_pos", 0))
+	{
+		case 0:
+			return GTK_POS_TOP;
+		case 1:
+			return GTK_POS_BOTTOM;
+		case 2:
+			return GTK_POS_LEFT;
+		case 3:
+			return GTK_POS_RIGHT;
+		default:
+			return GTK_POS_TOP;
+	}
+}
+
+static void roxterm_get_initial_tabs(ROXTermData *roxterm,
+		GtkPositionType *p_pos, int *p_num)
+{
+	*p_pos = roxterm_get_tab_pos(roxterm);
+	*p_num = options_lookup_int_with_default(roxterm->profile, "init_tabs", 1);
+}
+
 void roxterm_launch(void)
 {
 	ROXTermData *roxterm = g_new0(ROXTermData, 1);
+	GtkPositionType tabpos;
 	char *colour_scheme_name;
 	char *shortcut_scheme;
 	char *profile_name;
 	Options *shortcuts;
 	char *geom;
 	gboolean size_on_cli = FALSE;
+	int numtabs;
 
 	roxterm->zoom_factor = global_options_lookup_double("zoom");
 	if (!roxterm->zoom_factor)
@@ -2498,6 +2543,7 @@ void roxterm_launch(void)
 		profile_name, "roxterm profile");
 
 	roxterm_default_dimensions(roxterm);
+	roxterm_get_initial_tabs(roxterm, &tabpos, &numtabs);
 	geom = options_lookup_string(global_options, "geometry");
 	if (geom)
 	{
@@ -2574,7 +2620,8 @@ void roxterm_launch(void)
 	if (global_options_fullscreen)
 	{
 		global_options_fullscreen = FALSE;
-		multi_win_new_fullscreen(shortcuts, roxterm->zoom_index, roxterm);
+		multi_win_new_fullscreen(shortcuts, roxterm->zoom_index, roxterm,
+				numtabs, tabpos);
 	}
 	else
 	{
@@ -2594,7 +2641,8 @@ void roxterm_launch(void)
 						roxterm->geom_width, roxterm->geom_height);
 			}
 		}
-		multi_win_new_with_geom(shortcuts, roxterm->zoom_index, roxterm, geom);
+		multi_win_new_with_geom(shortcuts, roxterm->zoom_index, roxterm, geom,
+				numtabs, tabpos);
 	}
 	g_free(geom);
 
@@ -2613,9 +2661,10 @@ static void roxterm_tab_to_new_window(MultiWin *win, MultiTab *tab)
 	ROXTermData *match_roxterm;
 	MultiTab *match_tab = multi_win_get_current_tab(win);
 
+	roxterm = multi_tab_get_user_data(tab);
+	roxterm->win = win;
 	g_return_if_fail(match_tab);
 	match_roxterm = multi_tab_get_user_data(match_tab);
-	roxterm = multi_tab_get_user_data(tab);
 	if (roxterm == match_roxterm)
 	{
 		return;
@@ -2665,7 +2714,8 @@ void roxterm_init(void)
 		(MultiWinSizeFunc) roxterm_size_func,
 		roxterm_tab_to_new_window,
 		(MultiWinZoomHandler) roxterm_set_zoom_factor,
-		(MultiWinGetDisableMenuShortcuts) roxterm_get_disable_menu_shortcuts);
+		(MultiWinGetDisableMenuShortcuts) roxterm_get_disable_menu_shortcuts,
+		(MultiWinInitialTabs) roxterm_get_initial_tabs);
 	roxterm_launch();
 }
 
@@ -2673,14 +2723,17 @@ void roxterm_spawn(ROXTermData *roxterm, const char *command,
 	ROXTerm_SpawnType spawn_type)
 {
 	GError *error = NULL;
+	GtkPositionType tabpos;
+	int numtabs;
 
+	roxterm_get_initial_tabs(roxterm, &tabpos, &numtabs);
 	switch (spawn_type)
 	{
 		case ROXTerm_SpawnNewWindow:
 			roxterm->special_command = g_strdup(command);
 			roxterm->no_respawn = TRUE;
 			multi_win_new(multi_win_get_shortcut_scheme(roxterm->win),
-					roxterm->zoom_index, roxterm);
+				      roxterm->zoom_index, roxterm, numtabs, tabpos);
 			break;
 		case ROXTerm_SpawnNewTab:
 			roxterm->special_command = g_strdup(command);
